from __future__ import unicode_literals

from helium import Device, Configuration, DeviceConfiguration


def display_map(cls, client, uuid=False, include=None):
    def _trim_id(res):
        return res.id if uuid or client.uuid else res.short_id

    def _config(self):
        return _trim_id(self.configuration(use_included=True))

    def _device(self):
        return _trim_id(self.device(use_included=True))

    def _device_type(self):
        return self.device(use_included=True)._resource_type()

    def _loaded(self):
        return 'yes' if self.is_loaded else 'no'

    dict = super(DeviceConfiguration, cls).display_map(client, include=include)
    if include and Configuration in include:
        dict['config'] = _config
    if include and Device in include:
        dict['device'] = _device
        dict['type'] = _device_type

    dict['loaded'] = _loaded

    return dict


DeviceConfiguration.display_map = classmethod(display_map)



from __future__ import unicode_literals
from helium import Label, Sensor, Element
from operator import attrgetter


def display_map(cls, client, include=None):
    def _count_sensor(self):
        return len(self.sensors(use_included=True))

    def _count_element(self):
        return len(self.elements(use_included=True))

    dict = super(Label, cls).display_map(client)
    if include and Sensor in include:
        dict['sensors'] = _count_sensor
    if include and Element in include:
        dict['elements'] = _count_element
    dict.update([
        ('name', attrgetter('name'))
    ])
    return dict

Label.display_map = classmethod(display_map)



"""Metadata cli generator."""
import click

from helium_commander import Client, Metadata
from helium_commander import JSONParamType

pass_client = click.make_pass_decorator(Client)


def cli(cls, singleton=False):
    group = click.Group(name='metadata',
                        short_help="Commands on metadata.")
    resource_type = cls._resource_type()

    def _fetch_resource(client, id, **kwargs):
        if singleton:
            resource = cls.singleton(client)
        else:
            mac = kwargs.pop('mac', None)
            resource = cls.lookup(client, id, mac=mac)
        return resource

    def _id_argument():
        def wrapper(func):
            if not singleton:
                return click.argument('id', metavar=resource_type)(func)
            return func
        return wrapper

    @group.command('list')
    @_id_argument()
    @pass_client
    def _list(client, id=None, **kwargs):
        """Get metadata."""
        resource = _fetch_resource(client, id, **kwargs)
        metadata = resource.metadata()
        Metadata.display(client, [metadata], **kwargs)

    @group.command('update')
    @_id_argument()
    @click.argument('value', type=JSONParamType())
    @pass_client
    def _update(client, value, id=None, **kwargs):
        """Update metadata."""
        resource = _fetch_resource(client, id, **kwargs)
        metadata = resource.metadata()
        if value is not None:
            metadata = metadata.update(value)
        Metadata.display(client, [metadata], **kwargs)

    @group.command('replace')
    @_id_argument()
    @click.argument('value', type=JSONParamType())
    @pass_client
    def _replace(client, value, id=None, **kwargs):
        """Replace metadata."""
        resource = _fetch_resource(client, id, **kwargs)
        metadata = resource.metadata()
        if value is not None:
            metadata = metadata.replace(value)
        Metadata.display(client, [metadata], **kwargs)

    return group



from __future__ import unicode_literals
from helium import Metadata
from future.utils import iteritems


def display_map(cls, client, include=None):
    def _attributes(self):
        return {k: v for k, v in iteritems(vars(self))
                if not (k.startswith('_') or k == 'id')}

    dict = super(Metadata, cls).display_map(client, include=include)
    dict.update([
        ('attributes', _attributes),
    ])
    return dict

Metadata.display_map = classmethod(display_map)



import click
from helium_commander import Client, User
from collections import OrderedDict

pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on the authorized user.
    """
    pass


@cli.command()
@pass_client
def list(client):
    """Lists user information for the current user.

    Lists information associated withe the user for the current api key.
    """
    user = User.singleton(client)
    User.display(client, [user])


@cli.command()
@click.option('--name',
              help="the new name for the user")
@pass_client
def update(client, name):
    """Update the attributes of the user.

    Updates the attributes of the currently authorized user.
    """
    user = User.singleton(client)
    attributes = OrderedDict()
    if name:
        attributes['name'] = name
    user = user.update(attributes=attributes)
    User.display(client, [user])



from __future__ import unicode_literals
from helium import Element, Sensor


def display_map(cls, client, include=None):
    def _type(self):
        return getattr(self.meta, "device_type", None)

    def _mac(self):
        return getattr(self.meta, "mac", None)

    def _count_sensor(self):
        return len(self.sensors(use_included=True))

    def _seen(self):
        return getattr(self.meta, 'last_seen', None)

    def _connected(self):
        state = getattr(self.meta, 'connected', None)
        return 'yes' if state is True else 'no'

    def _name(self):
        return getattr(self, 'name', None)

    dict = super(Element, cls).display_map(client, include=include)
    dict.update([
        ('mac', _mac),
        ('type', _type)
    ])

    if include and Sensor in include:
        dict['sensors'] = _count_sensor

    dict.update([
        ('connected', _connected),
        ('seen', _seen),
        ('name', _name)
    ])
    return dict

Element.display_map = classmethod(display_map)



import click
from helium_commander import (
    Client,
    Label,
    Sensor,
    Element,
    device_sort_option,
    device_mac_option,
    metadata_filter_option,
    ResourceParamType
)
from helium_commander.commands import metadata, timeseries
from collections import namedtuple


pass_client = click.make_pass_decorator(Client)
label_includes = [Sensor, Element]

LabelActionResources = namedtuple('LabelResourceActions',
                                  ['add', 'remove', 'replace'])


def lookup_label_action_resources(client, cls, mac=False, **kwargs):
    """Look up resources for a label."""
    def _lookup(action, resources):
        id_reps = kwargs.pop(action, None)
        if not id_reps:
            return None         # No change
        if 'none' in id_reps:
            return []           # Empty out the resources
        return [cls.lookup(client, id, resources=resources, mac=mac)
                for id in id_reps]

    all_resources = cls.all(client)
    return LabelActionResources(_lookup('add', all_resources),
                                _lookup('remove', all_resources),
                                _lookup('replace', all_resources))


@click.group()
def cli():
    """Operations on labels of sensors.
    """
    pass


@cli.command()
@click.argument('label', required=False)
@metadata_filter_option
@pass_client
def list(client, label, **kwargs):
    """List labels.

    Lists information for a given LABEL or all labels in the
    organization.

    """
    if label:
        labels = [Label.lookup(client, label, include=label_includes)]
    else:
        metadata = kwargs.get('metadata') or None
        labels = Label.where(client, include=label_includes, metadata=metadata)
    Label.display(client, labels, include=label_includes)


cli.add_command(timeseries.cli(Label, history=False,
                               writable=False, device=False))


@cli.command()
@click.argument('name')
@click.option('--sensors',
              type=ResourceParamType(metavar='SENSOR'),
              help="Add sensors to a label")
@click.option('--elements',
              type=ResourceParamType(metavar='ELEMENT'),
              help="Add elements to a label")
@click.pass_context
def create(ctx, name, sensors, elements):
    """Create a label.

    Creates a label with a given NAME and an (optional) list of
    sensors and elements associated with that label.

    """
    client = ctx.find_object(Client)

    sensors = sensors or []
    if sensors:
        all_sensors = Sensor.all(client)
        sensors = [Sensor.lookup(client, id, resources=all_sensors)
                   for id in sensors]

    elements = elements or []
    if elements:
        all_elements = Element.all(client)
        elements = [Element.lookup(client, id, resources=all_elements)
                    for id in elements]

    label = Label.create(client, attributes={
        'name': name
    })

    if sensors:
        label.update_sensors(sensors)

    if elements:
        label.update_elements(elements)

    label = Label.find(client, label.id, include=label_includes)
    Label.display(client, [label], include=label_includes)


@cli.command()
@click.argument('label', nargs=-1)
@pass_client
def delete(client, label):
    """Delete one or more labels.

    Deletes the LABELs with the given ids
    """
    all_labels = Label.all(client)
    label = [Label.lookup(client, id, resources=all_labels) for id in label]
    for entry in label:
        entry.delete()
        click.echo("Deleted {} ".format(entry.id))


@cli.command()
@click.argument('label')
@click.option('--name',
              help="the new name for the label")
@pass_client
def update(client, label, name):
    """Update a label.

    Changes basic attributes on a label.

    To add or remove sensors or elements from a label see the `label
    element` and `label sensor` commands.

    """
    label = Label.lookup(client, label)
    if name:
        label.update(attributes={
            'name': name
        })

    label = Label.find(client, label.id, include=label_includes)
    Label.display(client, [label], include=label_includes)


cli.add_command(metadata.cli(Label))


@cli.command()
@click.argument('label')
@click.option('--add',
              type=ResourceParamType(metavar='SENSOR'),
              help="Add sensors to a label")
@click.option('--remove',
              type=ResourceParamType(metavar='SENSOR'),
              help="Remove sensors from a label")
@click.option('--replace',
              type=ResourceParamType(metavar='SENSOR'),
              help="Replace all sensors in a label")
@device_sort_option
@device_mac_option
@pass_client
def sensor(client, label, mac, **kwargs):
    """List sensors for a label.

    List sensors for a given LABEL.

    Add, remove or replace sensors from the LABEL by using the --add,
    --remove or --replace arguments respectively. Note that you can
    specify "none" with these to indicate an empty list.

    """
    label = Label.lookup(client, label)

    actions = lookup_label_action_resources(client, Sensor,
                                            mac=mac, **kwargs)

    if actions.add is not None:
        label.add_sensors(actions.add)

    if actions.remove is not None:
        label.remove_sensors(actions.remove)

    if actions.replace is not None:
        label.update_sensors(actions.replace)

    sensors = label.sensors()
    Sensor.display(client, sensors, **kwargs)


@cli.command()
@click.argument('label')
@click.option('--add',
              type=ResourceParamType(metavar='SENSOR'),
              help="Add sensors to a label")
@click.option('--remove',
              type=ResourceParamType(metavar='SENSOR'),
              help="Remove sensors from a label")
@click.option('--replace',
              type=ResourceParamType(metavar='SENSOR'),
              help="Replace all sensors in a label")
@device_sort_option
@device_mac_option
@pass_client
def element(client, label, mac, **kwargs):
    """List elements for a label.

    List elements for a given LABEL.

    Add, remove or replace sensors from the LABEL by using the --add,
    --remove or --replace arguments respectively. Note that you can
    specify "none" with these to indicate an empty list.

    """
    label = Label.lookup(client, label)

    actions = lookup_label_action_resources(client, Element,
                                            mac=mac, **kwargs)

    if actions.add is not None:
        label.add_elements(actions.add)

    if actions.remove is not None:
        label.remove_elements(actions.remove)

    if actions.replace is not None:
        label.update_elements(actions.replace)

    elements = label.elements()
    Element.display(client, elements, **kwargs)
